from menu.classes.theme import *

__all__=[
    'print_header'
]

# function that prints the terminal header when called
def print_header(program_name, page_name):
    if page_name != None:
        print(f' {color_theme.title} {program_name} {color_theme.end} {color_theme.text}{formatting.title_sep}{color_theme.end}{color_theme.page} {formatting.page_brackets[0]} {page_name} {formatting.page_brackets[1]}{color_theme.end}\n')
    else:
        print(f' {color_theme.title}  {program_name}  {color_theme.end}\n')




import keyboard as k
from time import sleep
from platform import python_version, system, python_version_tuple
from sys import setrecursionlimit

# import functions
from .functions.cls import *
from .functions.print_header import *
from .functions.clear_buffer import *
from .functions.debugprint import *

# import classes
from .classes.theme import *
from .classes.exceptions import *
from .classes.menu_options import *

version = '0.3.5'

__all__=[
    'menu',
    'version'
]

choice = 1
last_page = None
last_teme = ''
last_theme = ''

#selection_delay = 0.22  # 0.22 seems best
#refresh_rate = 0.01     # 0.01 seems appropriate

# menu master class
class menu(object):

    tick_rate = 0.01
    selection_delay = 0.30

    def __init__(self, program_name):
        self.name = program_name
        self.tick_rate = menu.tick_rate
        self.selection_delay = menu.selection_delay

    def generate(self, page_name:str, menu_items:list, menu_text:list|str = None, theme:str = None):

        '''
        generates an in-terminal selection menu that works both in windows command-prompt and powershell.

        '''

        # self setup
        #theme = theme
        #program_name = program_name
        #page_name = page_name
        #menu_items = menu_items
        #if menu_text != None:
        #    menu_text = menu_text

        global choice     
        global last_page
        global last_theme

        menu_text_list = []
        bList = False

        # this ensures that when changing pages, the selection returns to the top of the page
        if last_page != None:
            if last_page != page_name:
                choice = 1
        else:
            choice = 1
        last_page = page_name

        # only set the theme when applicable
        if last_theme != theme:
            if theme != None:
                color_theme.set_theme(theme)             
        last_theme = theme

        # get the total amount of menu_options loaded into the menu, minus any "_skip_" or empty string options.
        # if the choice goes above or below the number of loaded menu items, return to 1 or total.
        # this ensures that the options scroll in the desired way.
        total_options = 0
        invalid_options = 0
        valid_options = len(menu_items)
        for item in menu_items:
            if item in ['', ' ', '_skip_']:
                invalid_options += 1
        total_options = valid_options - invalid_options

        # allows menu to loop through the options
        if choice > total_options:
            choice = 1
        if choice < 1:
            choice = total_options

        # format menu_text input
        if menu_text != None:
            if isinstance(menu_text, str) == True: # if menu_text is string
                formatted_menu_text = f'{color_theme.text}{menu_text}{color_theme.end}'
            elif isinstance(menu_text, list) == True: # if menu_text is list
                bList = True
                for item in menu_text:
                    if item in ['_skip_', '', ' ']:
                        menu_text_list.append('')
                        continue
                    menu_text_list.append(f'{color_theme.text}{item}{color_theme.end}')
            else:
                raise menuException('menu_text *must* either be a string or a list')

        # if no menu options are present, raise exception (can't have a menu with no options, can you?).
        if len(menu_items) <= 0:
            raise Exception('menu_items *must* contain at least one menu item')
        

        clear_buffer()
        cls()                                   # clear terminal screen FIXME: this is causing flickering, there must be another way
        print_header(self.name, page_name)      # print the menu header
        if bList == False:
            print(f' {formatted_menu_text}')
        else:
            for item in menu_text_list:
                print(f' {item}')
        print('')

        # print menu items
        i = 1
        if isinstance(menu_items, list) == True:
            for item in menu_items:
                if item in ['', ' ', '_skip_']:
                    print('')
                    continue
                elif i == choice:
                    menu_option(i, item, True).print()
                else:
                    menu_option(i, item).print()
                i += 1
        else:
            raise menuException('menu_items *must* be a list')

        # l00p until user selects an item in the menu
        sleep(self.selection_delay)
        while not k.is_pressed('enter'):
            if k.is_pressed('s'):
                try:
                    choice += 1
                    return self.generate(page_name, menu_items, menu_text, theme)
                except:
                    return
            elif k.is_pressed('w'):
                try:
                    choice -= 1
                    return self.generate(page_name, menu_items, menu_text, theme)
                except:
                    return
            sleep(self.tick_rate)
        
        clear_buffer()
        return menu_option(choice, menu_items[choice-1], menu_items[choice-1]).select()

    def terminate(self):
        clear_buffer()
        cls()
        return

#check that user is using the correct OS and version of python
if system().lower() != 'windows':
    raise menuException(f'invalid operating system: "{system()}"\nmenu.py currently only works on windows machines.')
if int(python_version_tuple()[0]) < 3 or int(python_version_tuple()[1]) < 6:
    raise menuException(f'invalid python version: "{python_version()}"\nmenu.py makes use of features, such as f-strings, that require python 3.6 or newer.')

try:
    setrecursionlimit(10**6)
except:
    pass



from menu.classes.exceptions import *

try:
    from os import system, name
except:
    raise menuException(f'failed to import "os" module in {__name__}')

__all__=[
    'cls'
]

# function that clears the terminal screen when called
def cls():

    try:
        system('cls||clear')
    except:
        raise menuException(f'critical error in cls() function in {__name__}.py')

    return


from menu import *
from time import sleep

my_menu = menu('example_menu')
my_theme = "White"

def primary_menu():

    page_name = 'main menu'
    text = 'the default keybinds for navigation are "s" for DOWN, "w" for UP and "ENTER" for SELECT'
    menu_items = [
        'module information',                    #CASE 1
        'change theme via selection',            #CASE 2
        'change theme via input (unavailable)',  #CASE 3   *** CURRENTLY UNAVAILABLE ***
        '_skip_',                                # "_skip_", " " and "" can be used to create a blank line without messing up the selections
        '[X] exit'                               #CASE 4
    ]

    selection = my_menu.generate(page_name, menu_items, text, my_theme)

    match selection:
        case 1:
            return module_menu()
        case 2:
            return theme_selection_menu()
        case 3:
            return menuprint_error('this option is currently unavailable'), sleep(2)
        case 4:
            my_menu.terminate()
            return exit()

        #this should, in theory, never happen but we'll plan for it anyways
        case _: #wildcard
            menuprint_error('a critical error has occured'), sleep(5)
            my_menu.terminate()
            return exit()

def module_menu():

    page_name = 'module information'
    text = [
        'name: menu.py',
        'author: deadyeti (deadyeti@deadyeti.ca)',
        'created: 2021-OCT-12',
        'last updated: 2021-OCT-20',
        '_skip_',
        '--- description ---',
        '"a simple python package to quickly and easily generate in-terminal menus"'
    ]
    menu_items = [
        '[<] back',                #CASE 1
        '[X] exit'                 #CASE 2
    ]

    selection = my_menu.generate(page_name, menu_items, text, my_theme)

    match selection:
        case 1:
            return primary_menu()
        case 2:
            my_menu.terminate()
            return exit()

        #this should, in theory, never happen but we'll plan for it anyways
        case _:
            menuprint_error('a critical error has occured'), sleep(5)
            my_menu.terminate()
            return exit()

def theme_selection_menu():

    global my_theme

    '''
        *** please note that the color themes will be undergoing a rewrite and may not be working properly ***
    '''

    page_name = 'theme selection'
    text = 'please select one of the themes below:'

    menu_items = [
        'Red Theme',                 #CASE 1   
        'Green Theme',               #CASE 2  
        'Blue Theme',                #CASE 3
        'Light Blue Theme',          #CASE 4  
        'Yellow Theme',              #CASE 5  
        'White Theme',               #CASE 6
        ' ',                         # "_skip_", " " and "" can be used to create a blank line without messing up the selections
        '[<] back',                  #CASE 7
        '[X] exit'                   #CASE 8
    ]

    selection = my_menu.generate(page_name, menu_items, text, my_theme)

    match selection:
            case 1:
                my_theme = 'Red'
            case 2:
                my_theme = 'Green'
            case 3:
                my_theme = 'Blue'
            case 4:
                my_theme = 'Light Blue'
            case 5:
                my_theme = 'Yellow'
            case 6:
                my_theme = 'White'
            case 7:
                return primary_menu()
            case 8:
                my_menu.terminate()
                return exit()

            #this should, in theory, never happen but we'll plan for it anyways
            case _:
                menuprint_error('a critical error has occured'), sleep(5)
                my_menu.terminate()
                return exit()

    return theme_selection_menu()

while True:
    primary_menu()



from menu.classes.theme import *

__all__=[
    'debugprint'
]

bDebugging = True

def debugprint(msg:str, sleeptime:int, log = False):

    if bDebugging == True:

        from time import sleep

        if log == False:
            print(f'\n {color_theme.error}debug: {msg}{color_theme.end}'), sleep(sleeptime)
        else:
            pass

    else:
        pass

    return


from menu.classes.theme import *

__all__=[
    'menuprint',
    'menuprint_error'
]

# prints in color, using whichever color is set in the theme object
def menuprint(msg:str, newline:bool = True):

    '''
    A print() function that prints in whichever **text** color your menu is set to\n
    for **error** color printing, use menuprint_error()
    '''

    if newline == True:
        print(f'\n {color_theme.text}{msg}{color_theme.end}')
    else:
        print(f' {color_theme.text}{msg}{color_theme.end}')


# prints in error_color, using whichever color is set in the theme object
def menuprint_error(msg, newline:bool = False):

    '''
    a print() function that prints in whichever **error** color your menu is set to.\n
    for **text** color printing, user menuprint()
    '''

    if newline == False:
        print(f' {color_theme.error}*{msg}{color_theme.end}')
    else:
        print(f'\n {color_theme.error}*{msg}{color_theme.end}')


from setuptools import setup
import re

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

version = ''
with open('menu/menu.py') as f:
    version = re.search(r'^version\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('version is not set')

packages = [
    'menu',
    'menu/classes',
    'menu/functions'
]

setup(
    name="menu.py",
    version=version,
    install_requires=[
        'keyboard'
    ],
    author="deadyeti",
    author_email="deadyeti@deadyeti.ca",
    description="A simple python package for in-terminal menu creation on Windows",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/readyyeti/menu.py",
    project_urls={
        "Bug Tracker": "https://github.com/readyyeti/menu.py/issues",
    },
    packages=packages,
    classifiers=[
        "Natural Language :: English",
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
    ]
)
